/*
** flash.c - utility to program, erase and verify the Flash memory on
**           an ADM-XRC series card.
**
** Warning: Although this utility performs checks in order to ensure
**          that you do not attempt to program the Flash memory with a
**          bitstream for the wrong type of FPGA, ensure that you do
**          not program the flash with an invalid bitstream. Doing so
**          and powering up the FPGA for an extended period of time
**          can damage devices on the card.
**
** Utility program for ADM-XRC / ADM-XRC-P / ADM-XRCII / ADM-XRCIIPro-Lite
**
** (C) Copyright 2003 Alpha Data Parallel Systems Ltd.
*/

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <stdint.h>
#include <assert.h>
#include <ctype.h>
#include <string.h>

#include <admxrc2.h>
#include <admxrc2/flash.h>

#include "../common/common.h"

static ADMXRC2_HANDLE		card = ADMXRC2_HANDLE_INVALID_VALUE;
static ADMXRC2_FLASH_INFO	flashInfo;

static void
myStrlwr(char* s)
{
  char c;

  while ((c = *s) != '\0') {
    *s++ = tolower(c);
  }
}

int main(int argc, char *argv[])
{
  int               ret = 0;
  ADMXRC2_STATUS    status;
  ADMXRC2_CARD_INFO cardInfo;
  ADMXRC2_IMAGE     image = NULL;
  unsigned long     imageSize;
  unsigned char*    flash = NULL;
  char*             command;
  Arguments         arguments;
  Option            options[2];
  OptionType        paramType[2];
  char*             paramInfo[2];
  char*             paramMeaning[2];
  BOOLEAN           doChkblank, doErase, doProgram, doVerify;
  unsigned long     bootFlagAddr;
  unsigned long     imageAddr;
  int               nError = 0;

  options[0].key = "card";
  options[0].type = Option_uint;
  options[0].def.uintVal = 0;
  options[0].meaning = "ID of card to use";
  options[1].key = "index";
  options[1].type = Option_uint;
  options[1].def.uintVal = 0;
  options[1].meaning = "index of card to use";
  arguments.nOption = 2;
  arguments.option = options;
  arguments.nParamType = 0;
  arguments.minNParam = 0;

  paramType[0] = Option_string;
  paramInfo[0] = "command";
  paramMeaning[0] = "One of erase, program, verify or chkblank";
  paramType[1] = Option_path;
  paramInfo[1] = "file";
  paramMeaning[1] = "Bitstream filename";
  arguments.nParamType = sizeof(paramType) / sizeof(OptionType);
  arguments.minNParam = 1;
  arguments.paramType = paramType;
  arguments.paramInfo = paramInfo;
  arguments.paramMeaning = paramMeaning;

  parse_command_line(
		     "flash",
		     argc,
		     argv,
		     &arguments);

  command = arguments.param[0].stringVal;
  myStrlwr(command);
  doProgram = strcmp(command, "program") == 0;
  doVerify = strcmp(command, "verify") == 0;
  doErase = strcmp(command, "erase") == 0;
  doChkblank = strcmp(command, "chkblank") == 0;
  if (!doProgram && !doVerify && !doErase && !doChkblank) {
    printf("*** Please specify a valid command. Use ftest /? to get help.\n");
    ret = -1;
    goto done;
  }

  if (strcmp(command, "program") == 0 || strcmp(command, "verify") == 0) {
    if (arguments.nParam < 2) {
      printf("*** Must specify a file for programming or verification\n");
      ret = -1;
      goto done;
    }
  }

  if (options[0].specified && options[1].specified) {
    printf("*** Do not specify both /card and /index\n");
    ret = -1;
    goto done;
  }

  if (!options[1].specified) {
    ADMXRC2_CARDID cardID = options[0].value.uintVal;

    status = ADMXRC2_OpenCard(cardID, &card);
    if (status != ADMXRC2_SUCCESS) {
      printf("*** Failed to open card with ID %ld: %s\n",
	     (unsigned long) cardID, ADMXRC2_GetStatusString(status));
      ret = -1;
      goto done;
    }
  } else {
    unsigned int index = options[1].value.uintVal;

    status = ADMXRC2_OpenCardByIndex(index, &card);
    if (status != ADMXRC2_SUCCESS) {
      printf("*** Failed to open card with index %ld: %s\n",
	     (unsigned long) index, ADMXRC2_GetStatusString(status));
      ret = -1;
      goto done;
    }
  }

  ADMXRC2_GetCardInfo(card, &cardInfo);
  if (status != ADMXRC2_SUCCESS) {
    printf("*** Failed to get card info: %s\n", ADMXRC2_GetStatusString(status));
    ret = -1;
    goto done;
  }

  switch (cardInfo.BoardType) {
  case ADMXRC2_BOARD_ADMXRC:
  case ADMXRC2_BOARD_ADMXRC_P:
  case ADMXRC2_BOARD_ADMXRC2:
  case ADMXRC2_BOARD_ADMXRC2_PRO_LITE:
    break;

  default:
    printf("*** This example must be run on one of the following cards:\n\n");
    printf("\tADM-XRC\n");
    printf("\tADM-XRC-P\n");
    printf("\tADM-XRCII\n");
    printf("\tADM-XRCIIPro-Lite\n");
    ret = -1;
    goto done;
  }

  status = ADMXRC2_GetFlashInfo(card, &flashInfo);
  if (status != ADMXRC2_SUCCESS) {
    printf("*** Failed to get Flash information: %s\n",
	   ADMXRC2_GetStatusString(status));
    ret = -1;
    goto done;
  }

  switch (cardInfo.BoardType) {
  case ADMXRC2_BOARD_ADMXRC:
  case ADMXRC2_BOARD_ADMXRC_P:
  case ADMXRC2_BOARD_ADMXRC2:
    bootFlagAddr = flashInfo.UseableStart + 0x8000U;
    imageAddr = 0x8001U;
    break;

  case ADMXRC2_BOARD_ADMXRC2_PRO_LITE:
    bootFlagAddr = flashInfo.UseableStart;
    imageAddr = flashInfo.UseableStart + 0x20000U;
    break;

  default:
    assert(FALSE);
    break;
  }

  /* Set local bus clock to nominal frequency */
  status = ADMXRC2_SetClockRate(card, ADMXRC2_CLOCK_LCLK, 33.0e6, NULL);
  if (status != ADMXRC2_SUCCESS) {
    printf("Failed to set LCLK to %.1fMHz: %s\n",
	   33.0, ADMXRC2_GetStatusString(status));
    ret = -1;
    goto done;
  }
  
  if (doProgram || doVerify) {
    status = ADMXRC2_LoadBitstream(
				   card,
				   arguments.param[1].pathVal,
				   &image,
				   &imageSize);
    if (status != ADMXRC2_SUCCESS) {
      printf("*** Failed to load bitstream '%s': %s\n",
	     arguments.param[1].pathVal, ADMXRC2_GetStatusString(status));
      ret = -1;
      goto done;
    }
  } else {
    image = malloc(flashInfo.UseableLength);
    if (image == NULL) {
      printf("*** Failed to allocate 0x%lx bytes for image\n",
	     (unsigned long) flashInfo.UseableLength);
      ret = -1;
      goto done;
    }
    memset(image, 0xff, flashInfo.UseableLength);
  }

  printf("Flash type is %s, %ld(0x%lx) kB\n",
	 flashInfo.DeviceName,
	 (unsigned long) flashInfo.DeviceSize / 1024,
	 (unsigned long) flashInfo.DeviceSize / 1024);
  printf("Useable region is 0x%lx-0x%lx\n",
	 (unsigned long) flashInfo.UseableStart,
	 (unsigned long) flashInfo.UseableStart + flashInfo.UseableLength - 1);
  if (doProgram || doVerify) {
    printf("Bitstream length is %ld(0x%lx) bytes\n",
	   (unsigned long) imageSize, (unsigned long) imageSize);
  }
  printf("\n");

  if (doErase || doProgram) {
    unsigned long			i, limit;
    ADMXRC2_FLASHBLOCK_INFO	blockInfo;

    limit = flashInfo.UseableStart + flashInfo.UseableLength;
    printf("Erasing Flash from 0x%lx-0x%lx...\n",
	   (unsigned long) flashInfo.UseableStart, (unsigned long) limit - 1);
    i = flashInfo.UseableStart;
    while (i < limit) {
      status = ADMXRC2_GetFlashBlockInfo(card, i, &blockInfo);
      if (status != ADMXRC2_SUCCESS) {
	printf("*** Failed to get Flash block info for location 0x%lx: %s\n",
	       (unsigned long) i, ADMXRC2_GetStatusString(status));
	nError++;
	break;
      }

      printf("0x%lx\r", i);
      fflush(stdout);

      status = ADMXRC2_EraseFlash(card, blockInfo.Start, blockInfo.Length);
      if (status != ADMXRC2_SUCCESS) {
	printf("*** Failed to erase Flash at 0x%lx-0x%lx: %s\n",
	       (unsigned long) blockInfo.Start,
	       (unsigned long) blockInfo.Start + blockInfo.Length - 1,
	       ADMXRC2_GetStatusString(status));
	nError++;
	break;
      }

      i += blockInfo.Length;
    }

    if (nError) {
      printf("*** Erase failed - aborting.\n");
      ret = -1;
      goto done;
    }
  }
		
  if (doProgram) {
    uint8_t*                data;
    unsigned long           i, remaining, chunk;
    ADMXRC2_FLASHBLOCK_INFO blockInfo;

    /* Write the bitstream into the flash */
    if (!nError) {
      printf("Writing bitstream image at 0x%lx-0x%lx...\n",
	     imageAddr, imageAddr + imageSize - 1);

      i = imageAddr;
      remaining = imageSize;
      data = (uint8_t*) image;
      while (remaining) {
	status = ADMXRC2_GetFlashBlockInfo(card, i, &blockInfo);
	if (status != ADMXRC2_SUCCESS) {
	  printf("*** Failed to get Flash block info for location 0x%lx: %s\n",
		 (unsigned long) i, ADMXRC2_GetStatusString(status));
	  nError++;
	  break;
	}

	chunk = blockInfo.Length - (i - blockInfo.Start);
	if (chunk > remaining) {
	  chunk = remaining;
	}
	printf("0x%lx\r", i);
	fflush(stdout);

	status = ADMXRC2_WriteFlash(card, i, chunk, data);
	if (status != ADMXRC2_SUCCESS) {
	  printf("*** Failed to write Flash at 0x%lx-0x%lx: %s\n",
		 i, i + chunk - 1, ADMXRC2_GetStatusString(status));
	  nError++;
	  break;
	}

	i += chunk;
	remaining -= chunk;
	data += chunk;
      }
    }

    if (nError) {
      printf("*** Programming failed - aborting\n");
      ret = -1;
      goto done;
    }
  }
	
  if (1) {
    ADMXRC2_FLASHBLOCK_INFO blockInfo;
    unsigned long           verifyStart;
    unsigned long           verifyLength;
    unsigned long           i, j, remaining;
    unsigned long           pos;
    unsigned long           chunk;
    uint8_t*                buf;
    uint8_t*                img = (uint8_t*) image;

    if (doProgram || doVerify) {
      verifyStart = imageAddr;
      verifyLength = imageSize;
      printf("Verifying data from 0x%lx-0x%lx...\n",
	     verifyStart, verifyStart + verifyLength - 1);
    } else {
      verifyStart = flashInfo.UseableStart;
      verifyLength = flashInfo.UseableLength;
      printf("Blank checking from 0x%lx-0x%lx...\n",
	     verifyStart, verifyStart + verifyLength - 1);
    }

    flash = malloc(verifyLength);
    if (flash == NULL) {
      printf("*** Failed to allocate memory to read back flash\n");
      ret = -1;
      goto done;
    }

    buf = flash;
    remaining = verifyLength;
    pos = verifyStart;
    i = 0;
    while (i < verifyLength) {
      status = ADMXRC2_GetFlashBlockInfo(card, pos, &blockInfo);
      if (status != ADMXRC2_SUCCESS) {
	printf("*** Failed to get Flash block info for location 0x%lx: %s\n",
	       (unsigned long) i, ADMXRC2_GetStatusString(status));
	nError++;
	break;
      }

      chunk = blockInfo.Length - (pos - blockInfo.Start);
      if (chunk > remaining) {
	chunk = remaining;
      }
      printf("0x%lx\r", pos);
      fflush(stdout);

      status = ADMXRC2_ReadFlash(card, pos, chunk, buf);
      if (status != ADMXRC2_SUCCESS) {
	printf("*** Failed to read back Flash memory at 0x%lx-0x%lx: %s\n",
	       i, i + chunk - 1, ADMXRC2_GetStatusString(status));
	nError++;
	break;
      }

      for (j = 0; j < chunk; j++) {
	if (img[i + j] != buf[j]) {
	  if (nError < 10) {
	    printf("*** Error at Flash location 0x%lx: expected=0x%2.2lx, got=0x%2.2lx\n",
		   (unsigned long) (pos + j), (unsigned long) img[i + j], (unsigned long) buf[j]);
	  } else if (nError == 10) {
	    printf("*** (more errors)\n");
	  }
	  nError++;
	}
      }

      if (nError) {
	break;
      }

      i += chunk;
      remaining -= chunk;
      pos += chunk;
      buf += chunk;
    }

    if (nError) {
      printf("*** Flash %s failed with %ld data error(s)\n",
	     doChkblank ? "blank check" : "verification", (unsigned long) nError);
      goto done;
    } else {
      if (doProgram || doVerify) {
	printf("Flash is correct - bitstream verified.\n");
      } else {
	printf("Flash is correct - no bitstream present.\n");
      }
    }

    if (doProgram) {
      if (cardInfo.BoardType == ADMXRC2_BOARD_ADMXRC2_PRO_LITE) {
	uint32_t val32;

				/*
				** Set the flag that causes the FPGA to be loaded from flash,
				** and bitstream length.
				*/
	val32 = (imageSize - 1) << 8;
	printf("Setting load-on-reset flag at 0x%lx-0x%lx to 0x%8.8lx...\n",
	       (unsigned long) bootFlagAddr,
	       (unsigned long) bootFlagAddr + sizeof(val32) - 1,
	       (unsigned long) val32);
	status = ADMXRC2_WriteFlash(card, bootFlagAddr, sizeof(val32), &val32);
	if (status != ADMXRC2_SUCCESS) {
	  printf("*** Failed to set load-on-reset flag: %s\n", ADMXRC2_GetStatusString(status));
	  nError++;
	}
      } else {
	uint8_t val8;

				/* Set the flag that causes the FPGA to be loaded from flash */
	val8 = 0x0U;
	printf("Setting load-on-powerup flag at 0x%lx to 0x%2.2lx...\n",
	       (unsigned long) bootFlagAddr, (unsigned long) val8);
	status = ADMXRC2_WriteFlash(card, bootFlagAddr, 0x1U, &val8);
	if (status != ADMXRC2_SUCCESS) {
	  printf("*** Failed to set load-on-reset flag: %s\n", ADMXRC2_GetStatusString(status));
	  nError++;
	}
      }
    }

    if ((doProgram || doVerify) && !nError) {
      if (cardInfo.BoardType == ADMXRC2_BOARD_ADMXRC2_PRO_LITE) {
	uint32_t expected, bootFlag;

	expected = (imageSize - 1) << 8;
	status = ADMXRC2_ReadFlash(card, bootFlagAddr, sizeof(bootFlag), &bootFlag);
	if (status != ADMXRC2_SUCCESS) {
	  printf("*** Failed to read load-on-reset flag: %s\n",
		 ADMXRC2_GetStatusString(status));
	  nError++;
	} else {
	  if (bootFlag != expected) {
	    printf("*** Load-on-reset flag is 0x%8.8lx, should be 0x%8.8lx\n",
		   (unsigned long) bootFlag, (unsigned long) expected);
	    nError++;
	  }
	}
      } else {
	uint8_t expected, bootFlag;

	expected = 0x0U;
	status = ADMXRC2_ReadFlash(card, bootFlagAddr, 0x1U, &bootFlag);
	if (status != ADMXRC2_SUCCESS) {
	  printf("*** Failed to read back load-on-powerup flag: %s\n",
		 ADMXRC2_GetStatusString(status));
	  nError++;
	} else {
	  if (bootFlag != 0x0U) {
	    printf("*** Load-on-powerup flag is 0x%2lx, should be 0x%2lx\n",
		   (unsigned long) bootFlag, (unsigned long) expected);
	    nError++;
	  }
	}
      }

      if (nError) {
	printf("*** The bitstream may not be loaded on power up.\n");
	goto done;
      } else {
	printf("The bitstream will be loaded on power up.\n");
      }
    }
  }

 done:
  if (flash != NULL) {
    free(flash);
  }

  if (image != NULL) {
    if (doProgram || doVerify) {
      ADMXRC2_UnloadBitstream(image);
    } else {
      free(image);
    }
  }
	
  if (card != ADMXRC2_HANDLE_INVALID_VALUE) {
    ADMXRC2_CloseCard(card);
  }
   
  return ret;
}
