/*
** itest.c - example demonstrating interrupt wait functionality in
**           the ADM-XRC API via the ADMXRC2_WaitForInterrupt function.
**
** Example for ADM-XRC/ADM-XRC-P/ADM-XRC-IIL
**
** (C) Copyright 2000-2001 Alpha Data Parallel Systems Ltd.
*/

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <stdint.h>
#include <pthread.h>

#include <admxrc2.h>

#include "../common/common.h"

/* FPGA registers */
#define FPGA_IMASK	(0x0U)
#define FPGA_ISTAT	(0x4U)
#define FPGA_IARM	(0x8U)
#define FPGA_TEST	(0xcU)

static ADMXRC2_HANDLE card = ADMXRC2_HANDLE_INVALID_VALUE;

static unsigned long  signalled;

static pthread_t      int_thread;
static BOOLEAN        int_running = FALSE;
static BOOLEAN        terminate = FALSE;

static void*
interruptThread(void* arg)
{
  ADMXRC2_STATUS status;
  uint32_t       istat;
  uint32_t       dummy;

  printf("Interrupt thread started.\n");
	
  while (1) {
    status = ADMXRC2_WaitForInterrupt(card, NULL, 0, NULL);
    if (status != ADMXRC2_SUCCESS) {
      if (status != ADMXRC2_CANCELLED) {
	printf("*** Interrupt thread failed to wait for interrupt: %s\n",
	       ADMXRC2_GetStatusString(status));
	break;
      }
    }

    if (terminate) {
      break;
    }

    signalled++;

    /* Get the pending FPGA interrupts */
    ADMXRC2_Read(card,
		 ADMXRC2_IOWIDTH_32,
		 ADMXRC2_IOLOCAL,
		 FPGA_ISTAT,
		 &istat,
		 sizeof(istat));

    /* Clear the pending FPGA interrupts */
    ADMXRC2_Write(card,
		  ADMXRC2_IOWIDTH_32,
		  ADMXRC2_IOLOCAL,
		  FPGA_ISTAT,
		  &istat,
		  sizeof(istat));

    /* Rearm the FINTI_L line by forcing it high for 1 cycle */
    ADMXRC2_Write(card,
		  ADMXRC2_IOWIDTH_32,
		  ADMXRC2_IOLOCAL,
		  FPGA_IARM,
		  &dummy,
		  sizeof(dummy));
  }

  return (void*) 0;
}

int main(int argc, char *argv[])
{
  const double      clock_freq = 33.0e6;
  int               ret = 0;
  ADMXRC2_STATUS    status;
  ADMXRC2_CARD_INFO cardInfo;
  int               nint;
  const char*       filename;
  Arguments         arguments;
  Option            options[2];
  uint32_t          val32;

  options[0].key = "card";
  options[0].type = Option_uint;
  options[0].def.uintVal = 0;
  options[0].meaning = "ID of card to use";
  options[1].key = "index";
  options[1].type = Option_uint;
  options[1].def.uintVal = 0;
  options[1].meaning = "index of card to use";
  arguments.nOption = 2;
  arguments.option = options;
  arguments.nParamType = 0;
  arguments.minNParam = 0;

  parse_command_line("itest", argc, argv, &arguments);

  if (options[0].specified && options[1].specified) {
    printf("*** Do not specify both /card and /index\n");
    ret = -1;
    goto done;
  }

  if (!options[1].specified) {
    ADMXRC2_CARDID cardID = options[0].value.uintVal;

    status = ADMXRC2_OpenCard(cardID, &card);
    if (status != ADMXRC2_SUCCESS)
      {
	printf("*** Failed to open card with ID %ld: %s\n",
	       (unsigned long) cardID, ADMXRC2_GetStatusString(status));
	ret = -1;
	goto done;
      }
  } else {
    unsigned int index = options[1].value.uintVal;

    status = ADMXRC2_OpenCardByIndex(index, &card);
    if (status != ADMXRC2_SUCCESS) {
      printf("*** Failed to open card with index %ld: %s\n",
	     (unsigned long) index, ADMXRC2_GetStatusString(status));
      ret = -1;
      goto done;
    }
  }

  status = ADMXRC2_GetCardInfo(card, &cardInfo);
  if (status != ADMXRC2_SUCCESS) {
    printf("*** Failed to get card information: %s\n", ADMXRC2_GetStatusString(status));
    ret = -1;
    goto done;
  }
	
  switch (cardInfo.BoardType) {
  case ADMXRC2_BOARD_ADMXRC:
  case ADMXRC2_BOARD_ADMXRC_P:
  case ADMXRC2_BOARD_ADMXRC2_LITE:
  case ADMXRC2_BOARD_ADMXRC2:
  case ADMXRC2_BOARD_ADMXRC2_PRO_LITE:
    break;

  default:
    printf("*** This example must be run on one of the following cards:\n\n");
    printf("\tADM-XRC\n");
    printf("\tADM-XRC-P\n");
    printf("\tADM-XRCII-Lite\n");
    printf("\tADM-XRCII\n");
    printf("\tADM-XRCIIPro-Lite\n");
    ret = -1;
    goto done;
  }

  /* Set local bus clock to nominal frequency */
  status = ADMXRC2_SetClockRate(card, ADMXRC2_CLOCK_LCLK, clock_freq, NULL);
  if (status != ADMXRC2_SUCCESS) {
    printf("*** Failed to set LCLK to %.1fMHz: %s\n",
	   clock_freq / 1.0e6f, ADMXRC2_GetStatusString(status));
    ret = -1;
    goto done;
  }

  filename = get_fpga_filename(&cardInfo, "itest", &arguments);
  status = ADMXRC2_ConfigureFromFile(card, filename);
  if (status != ADMXRC2_SUCCESS) {
    printf("*** Failed to load the bitstream '%s': %s\n",
	   filename, ADMXRC2_GetStatusString(status));
    ret = -1;
    goto done;
  }

  /* Start up the thread that waits for interrupts. */
  signalled = 0;
  terminate = FALSE;
  if (pthread_create(&int_thread, NULL, interruptThread, NULL)) {
    printf("*** Failed to start interrupt thread\n");
    ret = -1;
    goto done;
  }
  int_running = TRUE;

  /* Enable interrupts in the FPGA */
  val32 = 0x0U;
  status = ADMXRC2_Write(card,
			 ADMXRC2_IOWIDTH_32,
			 ADMXRC2_IOLOCAL,
			 FPGA_IMASK,
			 &val32,
			 sizeof(val32));
  if (status != ADMXRC2_SUCCESS) {
    printf("*** Write to FPGA_IMASK failed: %s\n", ADMXRC2_GetStatusString(status));
    ret = -1;
    goto done;
  }

  /*
  ** Allow the user to generate interrupts by pressing enter.
  ** Ctrl-Z exits this loop.
  */
  nint = 0;
  while (1) {
    int      c;

    printf("Enter 'q' to quit, or anything else to generate an interrupt:\n");
    c = getchar();
    if (c == EOF || c == 'q' || c == 'Q') {
      break;
    }

    val32 = 0x1U;
    status = ADMXRC2_Write(card,
			   ADMXRC2_IOWIDTH_32,
			   ADMXRC2_IOLOCAL,
			   FPGA_TEST,
			   &val32,
			   sizeof(val32));
    if (status != ADMXRC2_SUCCESS) {
      printf("*** Write to FPGA_TEST failed: %s\n", ADMXRC2_GetStatusString(status));
    }
    nint++;
  }

  printf("Generated %ld interrupts\n", (unsigned long) nint);
  printf("Interrupt thread saw %ld interrupt(s)\n", (unsigned long) signalled);

 done:
  if (int_running) {
    /* Force the interrupt thread to terminate */
    terminate = TRUE;
    ADMXRC2_CancelWaitForInterrupt(card);
    pthread_join(int_thread, NULL);
  }

  if (card != ADMXRC2_HANDLE_INVALID_VALUE) {
    ADMXRC2_CloseCard(card);
  }
	
  return ret;
}
