/*
** simple.c - Trivial example showing how to implement read/write registers
**            in the FPGA.
**
** Example for ADM-XRC / ADM-XRC-P / ADM-XRCII-Lite / ADM-XRCII / ADM-XRCIIPro-Lite
**
** (C) Copyright 2000-2003 Alpha Data Parallel Systems Ltd.
*/

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <stdint.h>

#include <admxrc2.h>

#include "../common/common.h"

int main(int argc, char *argv[])
{
  int                ret = 0;
  ADMXRC2_STATUS     status;
  ADMXRC2_HANDLE     card = ADMXRC2_HANDLE_INVALID_VALUE;
  ADMXRC2_CARD_INFO  cardInfo;
  ADMXRC2_SPACE_INFO spInfo;
  volatile uint32_t* fpgaSpace;
  unsigned long      dataOut, dataIn;
  const char*        filename;
  Option             options[2];
  Arguments          arguments;

  /* Set up expected options */
  options[0].key = "card";
  options[0].type = Option_uint;
  options[0].def.uintVal = 0;
  options[0].meaning = "ID of card to use";
  options[1].key = "index";
  options[1].type = Option_uint;
  options[1].def.uintVal = 0;
  options[1].meaning = "index of card to use";
  arguments.nOption = 2;
  arguments.option = options;
  arguments.nParamType = 0;
  arguments.minNParam = 0;

  /* Parse command line */
  parse_command_line(
		     "simple",
		     argc,
		     argv,
		     &arguments);

  if (options[0].specified && options[1].specified) {
    printf("Do not specify both /card and /index\n");
    ret = -1;
    goto done;
  }

  if (!options[1].specified) {
    ADMXRC2_CARDID cardID = options[0].value.uintVal;

    status = ADMXRC2_OpenCard(cardID, &card);
    if (status != ADMXRC2_SUCCESS) {
      printf("Failed to open card with ID %ld: %s\n",
	     (unsigned long) cardID, ADMXRC2_GetStatusString(status));
      ret = -1;
      goto done;
    }
  } else {
    unsigned int index = options[1].value.uintVal;

    status = ADMXRC2_OpenCardByIndex(index, &card);
    if (status != ADMXRC2_SUCCESS) {
      printf("Failed to open card with index %ld: %s\n",
	     (unsigned long) index, ADMXRC2_GetStatusString(status));
      ret = -1;
      goto done;
    }
  }

  status = ADMXRC2_GetCardInfo(card, &cardInfo);
  if (status != ADMXRC2_SUCCESS) {
    printf("Failed to get card info: %s\n", ADMXRC2_GetStatusString(status));
    ret = -1;
    goto done;
  }

  switch (cardInfo.BoardType) {
  case ADMXRC2_BOARD_ADMXRC:
  case ADMXRC2_BOARD_ADMXRC_P:
  case ADMXRC2_BOARD_ADMXRC2_LITE:
  case ADMXRC2_BOARD_ADMXRC2:
  case ADMXRC2_BOARD_ADMXRC2_PRO_LITE:
    break;

  default:
    printf("This example must be run on one of the following cards:\n\n");
    printf("\tADM-XRC\n");
    printf("\tADM-XRC-P\n");
    printf("\tADM-XRCII-Lite\n");
    printf("\tADM-XRCII\n");
    printf("\tADM-XRCIIPro-Lite\n");
    ret = -1;
    goto done;
  }

  /* Get the address of FPGA space */
  status = ADMXRC2_GetSpaceInfo(card, 0, &spInfo);
  if (status != ADMXRC2_SUCCESS) {
    printf("Failed to get space 0 info: %s\n",
	   ADMXRC2_GetStatusString(status));
    ret = -1;
    goto done;
  }
  fpgaSpace = (volatile uint32_t*) spInfo.VirtualBase;

  /* Set local bus clock to nominal frequency */
  status = ADMXRC2_SetClockRate(card, ADMXRC2_CLOCK_LCLK, 33.0e6, NULL);
  if (status != ADMXRC2_SUCCESS) {
    printf("Failed to set LCLK to %.1fMHz: %s\n",
	   33.0e6, ADMXRC2_GetStatusString(status));
    ret = -1;
    goto done;
  }
	
  filename = get_fpga_filename(&cardInfo, "simple", &arguments);
  status = ADMXRC2_ConfigureFromFile(card, filename);
  if (status != ADMXRC2_SUCCESS) {
    printf ("Failed to load the bitstream '%s': %s\n",
	    filename, ADMXRC2_GetStatusString(status));
    ret = -1;
    goto done;
  }

  printf("====================\n" );
  printf("Enter values for I/O\n" );
  printf(" (use 55AA to exit)\n");
  printf("====================\n" );

  while (1) {
    scanf("%lx", &dataOut);
    fpgaSpace[0] = (uint32_t) dataOut;
    dataIn = (unsigned long) fpgaSpace[0];
    printf("Out = %8.8lx, In = %8.8lx\n",
	   (unsigned long) dataOut, (unsigned long) dataIn);
    if (dataOut == 0x55AAU) {
      break;
    }
  }

 done:
  if (card != ADMXRC2_HANDLE_INVALID_VALUE) {
    ADMXRC2_CloseCard(card);
  }
	
  return ret;
}
