--
-- plxsim_pkg.vhd - package "plxsim"
--
-- NOT SYNTHESISABLE
--
-- Package of components, procedures and functions for applying
-- stimulus to PLX local bus devices.
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

library std;
use std.textio.all;

package plxsim is

    constant plxsim_data_width : natural := 32;

    --
    -- An array of data words, intended to hold data to be written to the
    -- local bus or read from the local bus.
    --
    type plxsim_data_type is
        array(natural range <>) of std_logic_vector(plxsim_data_width - 1 downto 0);

    --
    -- Perform a direct slave read of a PLX device, with incrementing local
    -- bus address.
    --
    --     address     Local bus byte address.
    --
    --     byte_en     Local bus byte enables (must be constant over cycle).
    --
    --     data        Data read from local bus. Data'length indicates
    --                 desired number of words to read.
    --
    --     n           Returns the actual number of words read on the local
    --                 bus.
    --
    procedure plxsim_read(
               address    : in    natural;
               byte_en    : in    std_logic_vector;
               data       : inout plxsim_data_type;
               n          : out   natural;
        signal lreseto_l  : in    std_logic;
        signal lclk       : in    std_logic;
        signal la         : out   std_logic_vector;
        signal ld         : inout std_logic_vector;
        signal lbe_l      : out   std_logic_vector;
        signal lads_l     : out   std_logic;
        signal lblast_l   : out   std_logic;
        signal lwrite     : out   std_logic;
        signal lreadyi_l  : in    std_logic;
        signal lbterm_l   : in    std_logic;
        signal lholda     : in    std_logic);

    --
    -- Perform a direct slave write of a PLX device, with incrementing local
    -- bus address.
    --
    --     address     Local bus byte address.
    --
    --     byte_en     Local bus byte enables (must be constant over cycle).
    --
    --     data        Data to write to local bus. Data'length indicates
    --                 desired number of words to write.
    --
    --     n           Returns the actual number of words written on the local
    --                 bus.
    --
    procedure plxsim_write(
               address    : in    natural;
               byte_en    : in    std_logic_vector;
               data       : inout plxsim_data_type;
               n          : out   natural;
        signal lreseto_l  : in    std_logic;
        signal lclk       : in    std_logic;
        signal la         : out   std_logic_vector;
        signal ld         : inout std_logic_vector;
        signal lbe_l      : out   std_logic_vector;
        signal lads_l     : out   std_logic;
        signal lblast_l   : out   std_logic;
        signal lwrite     : out   std_logic;
        signal lreadyi_l  : in    std_logic;
        signal lbterm_l   : in    std_logic;
        signal lholda     : in    std_logic);

    --
    -- Perform a direct slave read of a PLX device, with constant local
    -- bus address.
    --
    --     address     Local bus byte address.
    --
    --     byte_en     Local bus byte enables (must be constant over cycle).
    --
    --     data        Data read from local bus. Data'length indicates
    --                 desired number of words to read.
    --
    --     n           Returns the actual number of words read on the local
    --                 bus.
    --
    procedure plxsim_read_const(
               address    : in    natural;
               byte_en    : in    std_logic_vector;
               data       : inout plxsim_data_type;
               n          : out   natural;
        signal lreseto_l  : in    std_logic;
        signal lclk       : in    std_logic;
        signal la         : out   std_logic_vector;
        signal ld         : inout std_logic_vector;
        signal lbe_l      : out   std_logic_vector;
        signal lads_l     : out   std_logic;
        signal lblast_l   : out   std_logic;
        signal lwrite     : out   std_logic;
        signal lreadyi_l  : in    std_logic;
        signal lbterm_l   : in    std_logic;
        signal lholda     : in    std_logic);

    --
    -- Perform a direct slave write of a PLX device, with constant local
    -- bus address.
    --
    --     address     Local bus byte address.
    --
    --     byte_en     Local bus byte enables (must be constant over cycle).
    --
    --     data        Data to write to local bus. Data'length indicates
    --                 desired number of words to write.
    --
    --     n           Returns the actual number of words written on the local
    --                 bus.
    --
    procedure plxsim_write_const(
               address    : in    natural;
               byte_en    : in    std_logic_vector;
               data       : inout plxsim_data_type;
               n          : out   natural;
        signal lreseto_l  : in    std_logic;
        signal lclk       : in    std_logic;
        signal la         : out   std_logic_vector;
        signal ld         : inout std_logic_vector;
        signal lbe_l      : out   std_logic_vector;
        signal lads_l     : out   std_logic;
        signal lblast_l   : out   std_logic;
        signal lwrite     : out   std_logic;
        signal lreadyi_l  : in    std_logic;
        signal lbterm_l   : in    std_logic;
        signal lholda     : in    std_logic);

    --
    -- Perform a direct slave read of a PLX device, with incrementing local
    -- bus address and demand-mode pausing.
    --
    --     address     Local bus byte address.
    --
    --     byte_en     Local bus byte enables (must be constant over cycle).
    --
    --     data        Data read from local bus. data'length indicates
    --                 desired number of words to read.
    --
    --     n           Returns the actual number of words read on the local bus.
    --                 This may be less than data'length if the slave pauses the
    --                 transfer by deasserting LDREQ#.
    --
    procedure plxsim_read_demand(
               address    : in    natural;
               byte_en    : in    std_logic_vector;
               data       : inout plxsim_data_type;
               n          : out   natural;
        signal lreseto_l  : in    std_logic;
        signal lclk       : in    std_logic;
        signal la         : out   std_logic_vector;
        signal ld         : inout std_logic_vector;
        signal lbe_l      : out   std_logic_vector;
        signal lads_l     : out   std_logic;
        signal lblast_l   : out   std_logic;
        signal lwrite     : out   std_logic;
        signal lreadyi_l  : in    std_logic;
        signal lbterm_l   : in    std_logic;
        signal ldreq_l    : in    std_logic;
        signal ldack_l    : out   std_logic;
        signal lholda     : in    std_logic);

    --
    -- Perform a direct slave write of a PLX device, with incrementing local
    -- bus address and demand-mode pausing.
    --
    --     address     Local bus byte address.
    --
    --     byte_en     Local bus byte enables (must be constant over cycle).
    --
    --     data        Data to write to local bus. data'length indicates
    --                 desired number of words to write.
    --
    --     n           Returns the actual number of words written on the local bus.
    --                 This may be less than data'length if the slave pauses the
    --                 transfer by deasserting LDREQ#.
    --
    procedure plxsim_write_demand(
               address    : in    natural;
               byte_en    : in    std_logic_vector;
               data       : inout plxsim_data_type;
               n          : out   natural;
        signal lreseto_l  : in    std_logic;
        signal lclk       : in    std_logic;
        signal la         : out   std_logic_vector;
        signal ld         : inout std_logic_vector;
        signal lbe_l      : out   std_logic_vector;
        signal lads_l     : out   std_logic;
        signal lblast_l   : out   std_logic;
        signal lwrite     : out   std_logic;
        signal lreadyi_l  : in    std_logic;
        signal lbterm_l   : in    std_logic;
        signal ldreq_l    : in    std_logic;
        signal ldack_l    : out   std_logic;
        signal lholda     : in    std_logic);

    --
    -- Perform a direct slave read of a PLX device, with constant local
    -- bus address and demand-mode pausing.
    --
    --     address     Local bus byte address.
    --
    --     byte_en     Local bus byte enables (must be constant over cycle).
    --
    --     data        Data read from local bus. data'length indicates
    --                 desired number of words to read.
    --
    --     n           Returns the actual number of words read on the local bus.
    --                 This may be less than data'length if the slave pauses the
    --                 transfer by deasserting LDREQ#.
    --
    procedure plxsim_read_const_demand(
               address    : in    natural;
               byte_en    : in    std_logic_vector;
               data       : inout plxsim_data_type;
               n          : out   natural;
        signal lreseto_l  : in    std_logic;
        signal lclk       : in    std_logic;
        signal la         : out   std_logic_vector;
        signal ld         : inout std_logic_vector;
        signal lbe_l      : out   std_logic_vector;
        signal lads_l     : out   std_logic;
        signal lblast_l   : out   std_logic;
        signal lwrite     : out   std_logic;
        signal lreadyi_l  : in    std_logic;
        signal lbterm_l   : in    std_logic;
        signal ldreq_l    : in    std_logic;
        signal ldack_l    : out   std_logic;
        signal lholda     : in    std_logic);

    --
    -- Perform a direct slave write of a PLX device, with constant local
    -- bus address and demand-mode pausing.
    --
    --     address     Local bus byte address.
    --
    --     byte_en     Local bus byte enables (must be constant over cycle).
    --
    --     data        Data to write to local bus. data'length indicates
    --                 desired number of words to write.
    --
    --     n           Returns the actual number of words written on the local bus.
    --                 This may be less than data'length if the slave pauses the
    --                 transfer by deasserting LDREQ#.
    --
    procedure plxsim_write_const_demand(
               address    : in    natural;
               byte_en    : in    std_logic_vector;
               data       : inout plxsim_data_type;
               n          : out   natural;
        signal lreseto_l  : in    std_logic;
        signal lclk       : in    std_logic;
        signal la         : out   std_logic_vector;
        signal ld         : inout std_logic_vector;
        signal lbe_l      : out   std_logic_vector;
        signal lads_l     : out   std_logic;
        signal lblast_l   : out   std_logic;
        signal lwrite     : out   std_logic;
        signal lreadyi_l  : in    std_logic;
        signal lbterm_l   : in    std_logic;
        signal ldreq_l    : in    std_logic;
        signal ldack_l    : out   std_logic;
        signal lholda     : in    std_logic);

end;

package body plxsim is
    
    procedure plxsim_report_violation(
        module    : in string;
        code      : in string;
        str       : in string)
    is
        constant str1 : string := "** At ";
        constant str2 : string := " in module ";
        constant str3 : string := "[";
        constant str4 : string := "] ";
        variable msg : line;
    begin
        write(msg, str1);
        write(msg, now);
        write(msg, str2);
        write(msg, module);
        write(msg, str3);
        write(msg, code);
        write(msg, str4);
        write(msg, str);
        writeline(output, msg);
    end;

    procedure plxsim_burst(
               read       : in    boolean;
               const_addr : in    boolean;
               address    : in    natural;
               byte_en    : in    std_logic_vector;
               data       : inout plxsim_data_type;
               nxfered    : out   natural;
        signal lreseto_l  : in    std_logic;
        signal lclk       : in    std_logic;
        signal la         : out   std_logic_vector;
        signal ld         : inout std_logic_vector;
        signal lbe_l      : out   std_logic_vector;
        signal lads_l     : out   std_logic;
        signal lblast_l   : out   std_logic;
        signal lwrite     : out   std_logic;
        signal lreadyi_l  : in    std_logic;
        signal lbterm_l   : in    std_logic;
        signal lholda     : in    std_logic)
    is
        variable data_len : natural := data'length;
        variable byte_en_width : natural := byte_en'length;
        variable ld_width : natural := ld'length;
        variable lbe_width : natural := lbe_l'length;

        type burst_state_type is
            (bs_arb,           -- waiting for 'lholda'
             bs_addr,          -- asserting 'lads_l'
             bs_data,          -- transferring data
             bs_turnaround);   -- turnaround cycle
        variable state : burst_state_type;
        variable n : natural;
        variable index : natural;
        variable blast_l : std_logic;
        variable addr : natural;
        variable finished : boolean;

    begin
        assert ld_width <= plxsim_data_width
            report "*** plxsim.plxsim_burst: ld too wide"
            severity failure;
        assert lbe_width * 8 = ld_width
            report "*** plxsim.plxsim_burst: ld_width / lbe_width mismatch"
            severity failure;
        assert lbe_width = byte_en_width
            report "*** plxsim.plxsim_burst: lbe_width / byte_en_width mismatch"
            severity failure;

        n := data_len;
        state := bs_arb;
        index := data'low;
        addr := address / 4;

        finished := false;
        while not finished loop

            wait until lclk'event and lclk = '1';

            case state is
                when bs_arb =>
                    if lholda = '1' then
                        state := bs_addr;
                        lads_l <= '0';
                        if read then
                            lwrite <= '0';
                        else
                            lwrite <= '1';
                        end if;
                        la <= EXT(CONV_STD_LOGIC_VECTOR(addr, la'length + 1), la'length);
                        if not const_addr then
                            addr := addr + 1;
                        end if;
                        lbe_l <= not byte_en;
                    end if;

                when bs_addr =>
                    state := bs_data;
                    lads_l <= '1';
                    if not read then
                        ld <= data(index)(ld'range);
                        index := index + 1;
                    end if;
                    if n = 1 then
                        blast_l := '0';
                        lblast_l <= '0';
                    else
                        blast_l := '1';
                        lblast_l <= '1';
                    end if;
                    n := n - 1;

                when bs_data =>
                    if (lreadyi_l = '0' or lbterm_l = '0') and blast_l = '0' then
                        state := bs_turnaround;
                        if read then
                            for i in lbe_l'range loop
                                if byte_en(i) = '1' then
                                    data(index)(8 * (i + 1) - 1 downto i * 8) :=
                                        ld(8 * (i + 1) - 1 downto i * 8);
                                else
                                    data(index)(8 * (i + 1) - 1 downto i * 8) := (others => 'U');
                                end if;
                            end loop;
                        end if;
                        if not read then
                            ld(ld'range) <= (others => 'Z');
                        end if;
                        lblast_l <= '1';
                    elsif lbterm_l = '0' or lreadyi_l = '0' then
                        if read then
                            for i in lbe_l'range loop
                                if byte_en(i) = '1' then
                                    data(index)(8 * (i + 1) - 1 downto i * 8) :=
                                        ld(8 * (i + 1) - 1 downto i * 8);
                                else
                                    data(index)(8 * (i + 1) - 1 downto i * 8) := (others => 'U');
                                end if;
                            end loop;
                            index := index + 1;
                        end if;

                        la <= EXT(CONV_STD_LOGIC_VECTOR(addr, la'length + 1), la'length);
                        if not const_addr then
                            addr := addr + 1;
                        end if;

                        if lbterm_l = '0' then
                            state := bs_addr;
                            lads_l <= '0';
                            ld(ld'range) <= (others => 'Z');
                        else
                            if not read then
                                ld(ld'range) <= data(index);
                                index := index + 1;
                            end if;
                            if n = 1 then
                                blast_l := '0';
                                lblast_l <= '0';
                            end if;
                            n := n - 1;
                        end if;
                    end if;

                when bs_turnaround =>
                    lads_l <= 'Z';
                    lblast_l <= 'Z';
                    lwrite <= 'Z';
                    la(la'range) <= (others => 'Z');
                    lbe_l(lbe_l'range) <= (others => 'Z');
                    finished := true;
            end case;

        end loop;

        nxfered := data_len - n;

    end procedure;

    procedure plxsim_burst_demand(
               read       : in    boolean;
               const_addr : in    boolean;
               demand     : in    boolean;
               address    : in    natural;
               byte_en    : in    std_logic_vector;
               data       : inout plxsim_data_type;
               nxfered    : out   natural;
        signal lreseto_l  : in    std_logic;
        signal lclk       : in    std_logic;
        signal la         : out   std_logic_vector;
        signal ld         : inout std_logic_vector;
        signal lbe_l      : out   std_logic_vector;
        signal lads_l     : out   std_logic;
        signal lblast_l   : out   std_logic;
        signal lwrite     : out   std_logic;
        signal lreadyi_l  : in    std_logic;
        signal lbterm_l   : in    std_logic;
        signal ldreq_l    : in    std_logic;
        signal ldack_l    : out   std_logic;
        signal lholda     : in    std_logic)
    is
        variable data_len : natural := data'length;
        variable byte_en_width : natural := byte_en'length;
        variable ld_width : natural := ld'length;
        variable lbe_width : natural := lbe_l'length;
        type burst_state_type is
            (bs_wait,          -- waiting for 'ldreq_l'
             bs_arb,           -- waiting for 'lholda'
             bs_addr,          -- asserting 'lads_l'
             bs_data,          -- transferring data
             bs_turnaround);   -- turnaround cycle
        variable state : burst_state_type;
        variable n : natural;
        variable index : natural;
        variable blast_l : std_logic;
        variable addr : natural;
        variable finished : boolean;

    begin
        assert ld_width <= plxsim_data_width
            report "*** plxsim.plxsim_burst: ld too wide"
            severity failure;
        assert lbe_width * 8 = ld_width
            report "*** plxsim.plxsim_burst: ld_width / lbe_width mismatch"
            severity failure;
        assert lbe_width = byte_en_width
            report "*** plxsim.plxsim_burst: lbe_width / byte_en_width mismatch"
            severity failure;

        n := data_len;
        index := data'low;
        addr := address / 4;

        if demand then
            state := bs_wait;
        else
            state := bs_arb;
        end if;

        finished := false;
        while not finished loop

            wait until lclk'event and lclk = '1';

            case state is
                when bs_wait =>
                    if ldreq_l = '0' then
                        state := bs_arb;
                    end if;

                when bs_arb =>
                    if lholda = '1' then
                        state := bs_addr;
                        lads_l <= '0';
                        if demand then
                            ldack_l <= '0';
                        end if;
                        if read then
                            lwrite <= '0';
                        else
                            lwrite <= '1';
                        end if;
                        la <= EXT(CONV_STD_LOGIC_VECTOR(addr, la'length + 1), la'length);
                        if not const_addr then
                            addr := addr + 1;
                        end if;
                        lbe_l <= not byte_en;
                    end if;

                when bs_addr =>
                    state := bs_data;
                    lads_l <= '1';
                    if not read then
                        ld <= data(index)(ld'range);
                        index := index + 1;
                    end if;
                    if n = 1 or (demand and ldreq_l = '1') then
                        blast_l := '0';
                        lblast_l <= '0';
                    else
                        blast_l := '1';
                        lblast_l <= '1';
                    end if;
                    n := n - 1;

                when bs_data =>
                    if (lreadyi_l = '0' and blast_l = '0') or lbterm_l = '0' then
                        if read then
                            data(index)(ld'range) := ld;
                        end if;
                        if not read then
                            ld(ld'range) <= (others => 'Z');
                        end if;
                        lblast_l <= '1';
                        state := bs_turnaround;
                        if demand then
                            ldack_l <= '1';
                        end if;
                    elsif lreadyi_l = '0' then
                        if read then
                            data(index)(ld'range) := ld;
                        end if;
                        if not read then
                            ld <= data(index);
                        end if;
                        la <= EXT(CONV_STD_LOGIC_VECTOR(addr, la'length + 1), la'length);
                        index := index + 1;
                        if not const_addr then
                            addr := addr + 1;
                        end if;
                        if n = 1 or (demand and ldreq_l = '1') then
                            blast_l := '0';
                            lblast_l <= '0';
                        end if;
                        n := n - 1;
                    end if;

                when bs_turnaround =>
                    lads_l <= 'Z';
                    lblast_l <= 'Z';
                    lwrite <= 'Z';
                    la(la'range) <= (others => 'Z');
                    lbe_l(lbe_l'range) <= (others => 'Z');
                    finished := true;
            end case;

        end loop;

        nxfered := data_len - n;

    end procedure;

    procedure plxsim_read(
               address    : in    natural;
               byte_en    : in    std_logic_vector;
               data       : inout plxsim_data_type;
               n          : out   natural;
        signal lreseto_l  : in    std_logic;
        signal lclk       : in    std_logic;
        signal la         : out   std_logic_vector;
        signal ld         : inout std_logic_vector;
        signal lbe_l      : out   std_logic_vector;
        signal lads_l     : out   std_logic;
        signal lblast_l   : out   std_logic;
        signal lwrite     : out   std_logic;
        signal lreadyi_l  : in    std_logic;
        signal lbterm_l   : in    std_logic;
        signal lholda     : in    std_logic)
    is begin
        plxsim_burst(
            true, false, address, byte_en, data, n,
            lreseto_l, lclk,
            la, ld, lbe_l, lads_l, lblast_l, lwrite, lreadyi_l, lbterm_l,
            lholda);
    end procedure;

    procedure plxsim_write(
               address    : in    natural;
               byte_en    : in    std_logic_vector;
               data       : inout plxsim_data_type;
               n          : out   natural;
        signal lreseto_l  : in    std_logic;
        signal lclk       : in    std_logic;
        signal la         : out   std_logic_vector;
        signal ld         : inout std_logic_vector;
        signal lbe_l      : out   std_logic_vector;
        signal lads_l     : out   std_logic;
        signal lblast_l   : out   std_logic;
        signal lwrite     : out   std_logic;
        signal lreadyi_l  : in    std_logic;
        signal lbterm_l   : in    std_logic;
        signal lholda     : in    std_logic)
    is begin
        plxsim_burst(
            false, false, address, byte_en, data, n,
            lreseto_l, lclk,
            la, ld, lbe_l, lads_l, lblast_l, lwrite, lreadyi_l, lbterm_l,
            lholda);
    end procedure;

    procedure plxsim_read_const(
               address    : in    natural;
               byte_en    : in    std_logic_vector;
               data       : inout plxsim_data_type;
               n          : out   natural;
        signal lreseto_l  : in    std_logic;
        signal lclk       : in    std_logic;
        signal la         : out   std_logic_vector;
        signal ld         : inout std_logic_vector;
        signal lbe_l      : out   std_logic_vector;
        signal lads_l     : out   std_logic;
        signal lblast_l   : out   std_logic;
        signal lwrite     : out   std_logic;
        signal lreadyi_l  : in    std_logic;
        signal lbterm_l   : in    std_logic;
        signal lholda     : in    std_logic)
    is begin
        plxsim_burst(
            true, true, address, byte_en, data, n,
            lreseto_l, lclk,
            la, ld, lbe_l, lads_l, lblast_l, lwrite, lreadyi_l, lbterm_l,
            lholda);
    end procedure;

    procedure plxsim_write_const(
               address    : in    natural;
               byte_en    : in    std_logic_vector;
               data       : inout plxsim_data_type;
               n          : out   natural;
        signal lreseto_l  : in    std_logic;
        signal lclk       : in    std_logic;
        signal la         : out   std_logic_vector;
        signal ld         : inout std_logic_vector;
        signal lbe_l      : out   std_logic_vector;
        signal lads_l     : out   std_logic;
        signal lblast_l   : out   std_logic;
        signal lwrite     : out   std_logic;
        signal lreadyi_l  : in    std_logic;
        signal lbterm_l   : in    std_logic;
        signal lholda     : in    std_logic)
    is begin
        plxsim_burst(
            false, true, address, byte_en, data, n,
            lreseto_l, lclk,
            la, ld, lbe_l, lads_l, lblast_l, lwrite, lreadyi_l, lbterm_l,
            lholda);
    end procedure;

    procedure plxsim_read_demand(
               address    : in    natural;
               byte_en    : in    std_logic_vector;
               data       : inout plxsim_data_type;
               n          : out   natural;
        signal lreseto_l  : in    std_logic;
        signal lclk       : in    std_logic;
        signal la         : out   std_logic_vector;
        signal ld         : inout std_logic_vector;
        signal lbe_l      : out   std_logic_vector;
        signal lads_l     : out   std_logic;
        signal lblast_l   : out   std_logic;
        signal lwrite     : out   std_logic;
        signal lreadyi_l  : in    std_logic;
        signal lbterm_l   : in    std_logic;
        signal ldreq_l    : in    std_logic;
        signal ldack_l    : out   std_logic;
        signal lholda     : in    std_logic)
    is begin
        plxsim_burst_demand(
            true, false, true, address, byte_en, data, n,
            lreseto_l, lclk,
            la, ld, lbe_l, lads_l, lblast_l, lwrite, lreadyi_l, lbterm_l,
            ldreq_l, ldack_l,
            lholda);
    end procedure;

    procedure plxsim_write_demand(
               address    : in    natural;
               byte_en    : in    std_logic_vector;
               data       : inout plxsim_data_type;
               n          : out   natural;
        signal lreseto_l  : in    std_logic;
        signal lclk       : in    std_logic;
        signal la         : out   std_logic_vector;
        signal ld         : inout std_logic_vector;
        signal lbe_l      : out   std_logic_vector;
        signal lads_l     : out   std_logic;
        signal lblast_l   : out   std_logic;
        signal lwrite     : out   std_logic;
        signal lreadyi_l  : in    std_logic;
        signal lbterm_l   : in    std_logic;
        signal ldreq_l    : in    std_logic;
        signal ldack_l    : out   std_logic;
        signal lholda     : in    std_logic)
    is begin
        plxsim_burst_demand(
            false, false, true, address, byte_en, data, n,
            lreseto_l, lclk,
            la, ld, lbe_l, lads_l, lblast_l, lwrite, lreadyi_l, lbterm_l,
            ldreq_l, ldack_l,
            lholda);
    end procedure;

    procedure plxsim_read_const_demand(
               address    : in    natural;
               byte_en    : in    std_logic_vector;
               data       : inout plxsim_data_type;
               n          : out   natural;
        signal lreseto_l  : in    std_logic;
        signal lclk       : in    std_logic;
        signal la         : out   std_logic_vector;
        signal ld         : inout std_logic_vector;
        signal lbe_l      : out   std_logic_vector;
        signal lads_l     : out   std_logic;
        signal lblast_l   : out   std_logic;
        signal lwrite     : out   std_logic;
        signal lreadyi_l  : in    std_logic;
        signal lbterm_l   : in    std_logic;
        signal ldreq_l    : in    std_logic;
        signal ldack_l    : out   std_logic;
        signal lholda     : in    std_logic)
    is begin
        plxsim_burst_demand(
            true, true, true, address, byte_en, data, n,
            lreseto_l, lclk,
            la, ld, lbe_l, lads_l, lblast_l, lwrite, lreadyi_l, lbterm_l,
            ldreq_l, ldack_l,
            lholda);
    end procedure;

    procedure plxsim_write_const_demand(
               address    : in    natural;
               byte_en    : in    std_logic_vector;
               data       : inout plxsim_data_type;
               n          : out   natural;
        signal lreseto_l  : in    std_logic;
        signal lclk       : in    std_logic;
        signal la         : out   std_logic_vector;
        signal ld         : inout std_logic_vector;
        signal lbe_l      : out   std_logic_vector;
        signal lads_l     : out   std_logic;
        signal lblast_l   : out   std_logic;
        signal lwrite     : out   std_logic;
        signal lreadyi_l  : in    std_logic;
        signal lbterm_l   : in    std_logic;
        signal ldreq_l    : in    std_logic;
        signal ldack_l    : out   std_logic;
        signal lholda     : in    std_logic)
    is begin
        plxsim_burst_demand(
            false, true, true, address, byte_en, data, n,
            lreseto_l, lclk,
            la, ld, lbe_l, lads_l, lblast_l, lwrite, lreadyi_l, lbterm_l,
            ldreq_l, ldack_l,
            lholda);
    end procedure;

end;
