--
-- sdkcomp.vhd - package "sdkcomp"
--
-- Package of components used by ADM-XRC SDK examples
--

library ieee;
use ieee.std_logic_1164.all;

package sdkcomp is

    -- PLX demand-mode DMA state machine
    component plxddsm
        port(
            clk:           in    std_logic;  -- Local bus clock
            rst:           in    std_logic;  -- Should reduce during synthesis to global reset
            sr:            in    std_logic;  -- Synchronous reset (do not assert unless bus is idle)
            qlads:         in    std_logic;  -- LADS# from local bus, qualified by address decoding
            ldack:         in    std_logic;  -- A particular bit of LDACK# from local bus, active high
            lblast:        in    std_logic;  -- LBLAST# from local bus, active high
            lwrite:        in    std_logic;  -- LWRITE from local bus
            ldreq:         out   std_logic;  -- To local bus, active high
            lreadyi:       out   std_logic;  -- To local bus, active high
            lreadyi_oe:    out   std_logic;  -- To local bus
            lbterm:        out   std_logic;  -- To local bus, active high
            lbterm_oe:     out   std_logic;  -- To local bus
            ld_oe:         out   std_logic;  -- Output enable for LD pins
            request:       in    std_logic;  -- Request for demand mode DMA
            burst:         in    std_logic;  -- Burst wanted/end burst
            transfer:      out   std_logic;  -- Data transfer on this clock edge
            decode:        out   std_logic;  -- Application should do further address decoding
            write:         out   std_logic); -- Cycle is a write cycle
    end component;

    -- PLX direct-master state machine
    component plxdmsm
        port(
            clk:           in    std_logic;  -- Local bus clock
            rst:           in    std_logic;  -- Asynchronous reset, should reduce to global reset
            sr:            in    std_logic;  -- Synchronous reset, must be asserted when bus idle
            request:       in    std_logic;  -- Pulse to request a direct-master cycle
            stop:          in    std_logic;  -- Pulse to terminate burst
            transfer:      out   std_logic;  -- Asserted 1 clock after data transfer takes place
            done:          out   std_logic;  -- Asserted when machine is idle
            write:         in    std_logic;  -- Assert coincident with 'request' for a write cycle
            lreadyo:       in    std_logic;  -- From local bus
            lbtermo:       in    std_logic;  -- From local bus
            fholda:        in    std_logic;  -- From local bus arbiter, FPGA's bus grant signal
            fhold:         out   std_logic;  -- To local bus arbiter, FPGA's bus request signal
            la_oe:         out   std_logic;  -- When asserted, app should enable LA pins
            lads:          out   std_logic;  -- To local bus LADS_L pin
            lads_oe:       out   std_logic;  -- When asserted, app should enable LADS_L pin
            lbe_oe:        out   std_logic;  -- When asserted, app should enable LBE_L pins
            lblast:        out   std_logic;  -- To local bus LBLAST_L pin
            lblast_oe:     out   std_logic;  -- When asserted, app should enable LBLAST_L pin
            ld_oe:         out   std_logic;  -- When asserted, app should enable LD pins
            lwrite:        out   std_logic;  -- To local bus LWRITE pin
            lwrite_oe:     out   std_logic); -- When asserted, app should enable LWRITE pin
    end component;

    -- PLX direct-slave state machine
    component plxdssm
        port(
            clk:           in    std_logic;  -- Local bus clock
            rst:           in    std_logic;  -- Should reduce during synthesis to FPGA global reset
            sr:            in    std_logic;  -- Synchronous reset (should never be asserted mid-burst)
            qlads:         in    std_logic;  -- LADS# signal qualified by address, FHOLDA and/or other decoding
            lblast:        in    std_logic;  -- Local bus LBLAST# signal
            lwrite:        in    std_logic;  -- Local bus LWRITE signal
            ld_oe:         out   std_logic;  -- Output enable for local bus data pins
            lreadyi:       out   std_logic;  -- To local bus LREADYI# signal
            lreadyi_oe:    out   std_logic;  -- To local bus LREADYI# signal
            lbterm:        out   std_logic;  -- To local bus LREADY# signal
            lbterm_oe:     out   std_logic;  -- To local bus LREADY# signal
            transfer:      out   std_logic;  -- Data transfer on this clock edge
            decode:        out   std_logic;  -- Application should do further address decoding
            write:         out   std_logic;  -- 'lwrite' latched on 'qlads'
            ready:         in    std_logic;  -- Apply pulse when ready
            stop:          in    std_logic); -- Assert to terminate burst
    end component;

    -- Synchronous 32-bit wide FIFO
    component sfifo
        generic(
            n_full:        in    natural:= 4;  -- FIFO is nearly-full when n_full or less free locations
            n_empty:       in    natural:= 4); -- FIFO is nearly-empty when n_empty or less used locations
        port(
            clk:           in    std_logic;                      -- Clock
            rst:           in    std_logic;                      -- Asynchronous reset (reduces to global reset)
            sr:            in    std_logic;                      -- Synchronous reset
            d:             in    std_logic_vector(31 downto 0);  -- Data in
            q:             out   std_logic_vector(31 downto 0);  -- Data out
            w:             in    std_logic;                      -- FIFO write command
            we:            out   std_logic;                      -- Write-side empty flag
            wnf:           out   std_logic;                      -- Write-side nearly-full flag
            wf:            out   std_logic;                      -- Write-side full flag
            r:             in    std_logic;                      -- FIFO read command
            re:            out   std_logic;                      -- Read-side empty flag
            rne:           out   std_logic;                      -- Read-side nearly-empty flag
            rf:            out   std_logic;                      -- Read-side full flag
            status:        out   std_logic_vector(34 downto 0)); -- Status vector (for debugging)
    end component;

end;
