--
-- sfifo.vhd - Synchronous 32-bit wide FIFO
--
-- Modules defined:
--
--    sfifo                 512 word by 32-bit FIFO using BlockRAMs
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

entity sfifo is
    generic(
        n_full:  in    natural := 4;  -- FIFO is nearly-full when n_full or less free locations
        n_empty: in    natural := 4); -- FIFO is nearly-empty when n_empty or less used locations
    port(
        clk:     in    std_logic;                      -- Clock
        rst:     in    std_logic;                      -- Reduces to global reset
        sr:      in    std_logic;                      -- Synchronous reset
        d:       in    std_logic_vector(31 downto 0);  -- Data in
        q:       out   std_logic_vector(31 downto 0);  -- Data out
        w:       in    std_logic;                      -- FIFO write command
        we:      out   std_logic;                      -- Write-side empty flag
        wnf:     out   std_logic;                      -- Write-side nearly-full flag
        wf:      out   std_logic;                      -- Write-side full flag
        r:       in    std_logic;                      -- FIFO read command
        re:      out   std_logic;                      -- Read-side empty flag
        rne:     out   std_logic;                      -- Read side nearly-empty flag
        rf:      out   std_logic;                      -- Read side full flag
        status:  out   std_logic_vector(34 downto 0)); -- Status vector (for debugging)
end sfifo;

architecture struct of sfifo is

    constant n_full_vec  : natural := 512 - n_full;
    constant n_empty_vec : natural := n_empty;

    signal da:        std_logic_vector(8 downto 0);
    signal qa:        std_logic_vector(8 downto 0);
    signal size:      std_logic_vector(9 downto 0);
    
    signal i_we:      std_logic;
    signal i_wnf:     std_logic;
    signal i_wf:      std_logic;
    signal i_re:      std_logic;
    signal i_rne:     std_logic;
    signal i_rf:      std_logic;
    
    signal r1e:       std_logic;
    signal wq:        std_logic;
    
    signal push:      std_logic;
    signal enb:       std_logic;
    signal wea:       std_logic;
    
    component sfiforam
        port(
            clk:        in  std_logic;
            rst:        in  std_logic;
            sr:         in  std_logic;
            da:         in  std_logic_vector(8 downto 0);
            d:          in  std_logic_vector(31 downto 0);
            qa:         in  std_logic_vector(8 downto 0);
            q:          out std_logic_vector(31 downto 0);
            r:          in  std_logic;
            w:          in  std_logic);
    end component;
        
begin

    rf     <= i_rf;
    rne    <= i_rne;
    re     <= i_re;
    wf     <= i_wf;
    wnf    <= i_wnf;
    we     <= i_we;
    status <= qa & da & size & r1e & i_rf & i_rne & i_re & i_wf & i_wnf & i_we;
    
    push   <= wq and i_re;
    enb    <= push or (r and not r1e);
    wea    <= w and not i_wf;
    
    update_flags: process(clk, rst)
    begin
        if rst = '1' then
            size <= (others => '0');
            i_we <= '1';
            i_wnf <= '0';
            i_wf <= '0';
            i_re <= '1';
            i_rne <= '1';
            i_rf <= '0';
            r1e <= '1';
            da <= (others => '0');
            qa <= (others => '0');
            wq <= '0';
        elsif clk'event and clk = '1' then
            if sr = '1' then
                size <= (others => '0');
                i_we <= '1';
                i_wnf <= '0';
                i_wf <= '0';
                i_re <= '1';
                i_rne <= '1';
                i_rf <= '0';
                r1e <= '1';
                da <= (others => '0');
                qa <= (others => '0');
                wq <= '0';
            else
                --
                -- writer side
                --

                if w = '1' and i_wf = '0' then
                    da <= da + '1';
                end if;

                if w = '1' and r = '0' then
                    i_we <= '0';
                elsif w = '0' and r = '1' and size = 1 then
                    i_we <= '1';
                end if;

                if w = '1' and r = '0' and size = 511 then
                    i_wf <= '1';
                elsif w = '0' and r = '1' and i_wf = '1' then
                    i_wf <= '0';
                end if;

                if w = '1' and r = '0' and size = (n_full_vec - 1) then
                    i_wnf <= '1';
                elsif w = '0' and r = '1' and size = n_full_vec then
                    i_wnf <= '0';
                end if;
                
                --
                -- Register the qualified write signal into 'wq' - this is
                -- used to push the data through when the FIFO read empty
                -- flag changes to 0 (see the 'push' signal above).
                --
                wq <= w and not i_wf;
                
                --
                -- reader side
                --

                if enb = '1' then
                    qa <= qa + '1';
                end if;

                if r = '1' and w = '0' then
                    i_rf <= '0';
                elsif r = '0' and w = '1' and size = 511 then
                    i_rf <= '1';
                end if;

                if r = '1' and r1e = '1' then
                    i_re <= '1';
                elsif r = '0' and wq = '1' and i_re = '1' then
                    i_re <= '0';
                end if;

                if r = '1' and w = '0' and size = 2 then
                    r1e <= '1';
                elsif r = '0' and w = '1' and size = 1 then
                    r1e <= '0';
                end if;

                if r = '1' and w = '0' and size = (n_empty_vec + 1) then
                    i_rne <= '1';
                elsif r = '0' and w = '1' and size = n_empty_vec then
                    i_rne <= '0';
                end if;
                
                --
                -- FIFO level counter
                --
                if w = '1' and r = '0' and i_wf = '0' then
                    size <= size + 1;
                elsif r = '1' and w = '0' and i_re = '0' then
                    size <= size - 1;
                end if;
            end if;
        end if;
    end process;

    sfiforam0 : sfiforam
        port map(
            clk => clk,
            rst => rst,
            sr  => sr,
            da  => da,
            d   => d,
            qa  => qa,
            q   => q,
            r   => enb,
            w   => wea);
    
end struct;
